@@ -64,17 +64,6 @@
 #include "PluginManager.h"
 #include "SignalWidget.h" // for MouseMode
 
-//#include "toolbar/filenew.xpm"
-//#include "toolbar/fileopen.xpm"
-//#include "toolbar/filefloppy.xpm"
-//#include "toolbar/filesaveas.xpm"
-
-//#include "toolbar/editcut.xpm"
-//#include "toolbar/editcopy.xpm"
-//#include "toolbar/editpaste.xpm"
-//#include "toolbar/eraser.xpm"
-//#include "toolbar/delete.xpm"
-
 #include "pics/playback_loop.xpm"
 #include "pics/playback_pause.xpm"
 #include "pics/playback_pause2.xpm"
@@ -307,13 +296,13 @@ TopWidget::TopWidget(KwaveApp &main_app)
 	i18n("insert the content of clipboard"));
 
     m_toolbar->insertButton(
-	icon_loader.loadIcon("eraser.png", KIcon::Toolbar),
+	icon_loader.loadIcon("editclear.png", KIcon::Toolbar),
 	-1, SIGNAL(clicked()),
 	this, SLOT(toolbarEditErase()), true,
 	i18n("mute the current selection"));
 
     m_toolbar->insertButton(
-	icon_loader.loadIcon("edittrash.png", KIcon::Toolbar),
+	icon_loader.loadIcon("editdelete.png", KIcon::Toolbar),
 	-1, SIGNAL(clicked()),
 	this, SLOT(toolbarEditDelete()), true,
 	i18n("delete the current selection"));
@@ -422,6 +411,7 @@ TopWidget::TopWidget(KwaveApp &main_app)
 
     int h = m_zoomselect->sizeHint().height();
     m_zoomselect->setFocusPolicy(QWidget::NoFocus);
+    m_zoomselect->setMinimumWidth(h*5);
 
     m_toolbar->setMinimumHeight(max(m_zoomselect->sizeHint().height()+2,
 	m_toolbar->sizeHint().height()));
@@ -776,10 +766,14 @@ int TopWidget::revert()
 //***************************************************************************
 bool TopWidget::closeFile()
 {
-    ThreadsafeX11Guard x11_guard;
+    if (m_plugin_manager && m_plugin_manager->onePluginRunning()) {
+	qWarning("TopWidget::closeFile() - currently not possible, "\
+	         "a plugin is running :-(");
+	return false;
+    }
 
-    Q_ASSERT(m_main_widget);
     if (signalManager().isModified()) {
+	ThreadsafeX11Guard x11_guard;
 	int res =  KMessageBox::warningYesNoCancel(this,
 	    i18n("This file has been modified.\nDo you want to save it?"));
 	if (res == KMessageBox::Cancel) return false;
@@ -791,6 +785,13 @@ bool TopWidget::closeFile()
 	}
     }
 
+    // close all plugins that still might use the current signal
+    if (m_plugin_manager) {
+	m_plugin_manager->sync();
+	m_plugin_manager->signalClosed();
+    }
+
+    Q_ASSERT(m_main_widget);
     if (m_main_widget) m_main_widget->closeSignal();
 
     m_url = KURL();
@@ -866,7 +867,7 @@ int TopWidget::saveFile()
 
 	// if saving in current format is not possible (no encoder),
 	// then try to "save/as" instead...
-	if (res == -EMEDIUMTYPE) res = saveFileAs(false);
+	if (res == -EINVAL) res = saveFileAs(false);
     } else res = saveFileAs(false);
 
     updateCaption();
