@@ -21,7 +21,7 @@
 #include <qdir.h>
 #include <qimage.h>
 #include <qkeycode.h>
-#include <qpntarry.h>
+#include <qpointarray.h>
 #include <qpushbutton.h>
 #include <qtimer.h>
 
@@ -45,11 +45,11 @@ LogoWidget::LogoWidget(QWidget *parent, const char *name)
     m_timer = 0;
 
     m_img = new QPixmap(xpm_aboutlogo);
-    ASSERT(m_img);
+    Q_ASSERT(m_img);
     if (!m_img) return;
 
     m_timer = new QTimer(this);
-    ASSERT(m_timer);
+    Q_ASSERT(m_timer);
     if (!m_timer) return;
     connect(m_timer, SIGNAL(timeout()), this, SLOT(doAnim()));
 
