@@ -25,7 +25,7 @@
 KWAVE_PLUGIN(AudiofileCodecPlugin,"codec_audiofile","Thomas Eschenbacher");
 
 /***************************************************************************/
-AudiofileCodecPlugin::AudiofileCodecPlugin(PluginContext &c)
+AudiofileCodecPlugin::AudiofileCodecPlugin(const PluginContext &c)
     :KwavePlugin(c), m_decoder(0)
 {
     i18n("codec_audiofile");
