@@ -40,8 +40,6 @@
 #include "libkwave/SampleFormat.h"
 #include "libkwave/SampleWriter.h"
 
-#include "libgui/Notice.h"
-
 #include "kwave/PluginManager.h"
 #include "kwave/SignalManager.h"
 #include "kwave/TopWidget.h"
@@ -135,6 +133,8 @@ QStringList *RecordPlugin::setup(QStringList &previous_params)
     m_controller.enableTrigger(m_dialog->params().record_trigger_enabled);
     connect(m_dialog, SIGNAL(sigPreRecordingChanged(bool)),
             &m_controller, SLOT(enablePrerecording(bool)));
+    connect(m_dialog, SIGNAL(sigPreRecordingChanged(bool)),
+            this, SLOT(prerecordingChanged(bool)));
     m_controller.enablePrerecording(m_dialog->params().pre_record_enabled);
 
     // connect the record controller and this
@@ -200,9 +200,7 @@ QStringList *RecordPlugin::setup(QStringList &previous_params)
 void RecordPlugin::notice(QString message)
 {
     Q_ASSERT(m_dialog);
-    if (!m_dialog) return;
-
-    (void) new Notice(m_dialog, message);
+    if (m_dialog) m_dialog->message(message);
 }
 
 //***************************************************************************
@@ -316,7 +314,7 @@ void RecordPlugin::setDevice(const QString &device)
     if (!m_dialog || !m_device) return;
 
     InhibitRecordGuard _lock(*this); // don't record while settings change
-    qDebug("RecordPlugin::setDevice(%s)", device.local8Bit().data());
+    qDebug("RecordPlugin::setDevice('%s')", device.local8Bit().data());
 
     // open and initialize the device
     int result = m_device->open(device);
@@ -338,18 +336,55 @@ void RecordPlugin::setDevice(const QString &device)
     }
 
     if (result < 0) {
-	qWarning("RecordPlugin::openDevice(): "\
-	         "opening the device failed.");
+	qWarning("RecordPlugin::openDevice('%s'): "\
+	         "opening the device failed. error=%d",
+	         device.local8Bit().data(), result);
 
-// 	// show an error message box
-// 	KMessageBox::error(parentWidget(), result,
-// 	    i18n("unable to open '%1'").arg(
-// 	    m_device_name));
+	m_controller.setInitialized(false);
+	m_dialog->showDevicePage();
+
+	if (m_device_name.length()) {
+	    // build a short device name for showing to the user
+	    QString short_device_name = m_device_name;
+	    if (m_device_name.contains("|")) {
+		// tree syntax: extract card + device
+		short_device_name = m_device_name.section("|", 0, 0) +
+		    ", " + m_device_name.section("|", 3, 3);
+	    }
+
+	    // show an error message box
+	    QString reason;
+	    switch (result) {
+		case -ENOENT:
+		case -ENODEV:
+		case -ENXIO:
+		case -EIO:
+		    reason = i18n(
+			"Kwave was unable to open the device '%1'.\n"\
+			"Maybe your system lacks support for the corresponding "\
+			" hardware or the hardware is not connected."
+		    ).arg(short_device_name);
+		    break;
+		case -EBUSY:
+		    reason = i18n(
+			"The device '%1' seems to be occupied by another "\
+			"application.\n"\
+			"Please try again later."
+		    ).arg(short_device_name);
+		    break;
+	    }
+
+	    if (reason.length()) KMessageBox::sorry(parentWidget(), reason,
+		i18n("unable to open the recording device"));
+	}
+
+	m_device_name = QString::null;
 	changeTracks(0);
-	return;
+    } else {
+	m_controller.setInitialized(true);
+	changeTracks(m_dialog->params().tracks);
     }
 
-    changeTracks(m_dialog->params().tracks);
 }
 
 //***************************************************************************
@@ -359,10 +394,11 @@ void RecordPlugin::changeTracks(unsigned int new_tracks)
     if (!m_dialog) return;
 
     InhibitRecordGuard _lock(*this); // don't record while settings change
-    qDebug("RecordPlugin::changeTracks(%u)", new_tracks);
+//     qDebug("RecordPlugin::changeTracks(%u)", new_tracks);
 
-    if (!m_device || !new_tracks) {
+    if (!m_device || m_device_name.isNull()) {
 	// no device -> dummy/shortcut
+	m_dialog->setSupportedTracks(0,0);
 	m_dialog->setTracks(0);
 	changeSampleRate(0);
 	return;
@@ -396,8 +432,7 @@ void RecordPlugin::changeTracks(unsigned int new_tracks)
 		    s2 = i18n("%1 tracks").arg(tracks);
 	    }
 
-	    notice(i18n("This device does not support recording with %1, "\
-		        "using %2 instead.").arg(s1).arg(s2));
+	    notice(i18n("%1 is not supported, using %2").arg(s1).arg(s2));
 	}
     }
     m_dialog->setSupportedTracks(min, max);
@@ -409,7 +444,7 @@ void RecordPlugin::changeTracks(unsigned int new_tracks)
 	tracks = m_device->tracks();
 	if (new_tracks && (tracks > 0)) notice(
 	    i18n("Recording with %1 track(s) failed, "\
-		 "using %2 track(s) instead.").arg(new_tracks).arg(tracks));
+		 "using %2 track(s)").arg(new_tracks).arg(tracks));
     }
     m_dialog->setTracks(tracks);
 
@@ -426,10 +461,10 @@ void RecordPlugin::changeSampleRate(double new_rate)
     InhibitRecordGuard _lock(*this); // don't record while settings change
 //     qDebug("RecordPlugin::changeSampleRate(%u)", (unsigned int)new_rate);
 
-    if (!m_device || (new_rate <= 0)) {
+    if (!m_device || m_device_name.isNull()) {
 	// no device -> dummy/shortcut
 	m_dialog->setSampleRate(0);
-	changeCompression(0);
+	changeCompression(-1);
 	return;
     }
 
@@ -441,7 +476,7 @@ void RecordPlugin::changeSampleRate(double new_rate)
 	double nearest = supported_rates.last();
 	QValueList<double>::Iterator it;
 	for (it=supported_rates.begin(); it != supported_rates.end(); ++it) {
-	    if (fabs(*it - nearest) <= fabs(rate - nearest))
+	    if (fabs(*it - rate) <= fabs(nearest - rate))
 	        nearest = *it;
 	}
 	rate = nearest;
@@ -450,8 +485,8 @@ void RecordPlugin::changeSampleRate(double new_rate)
 	const QString sr2(m_dialog->rate2string(rate));
 	if (((int)new_rate > 0) && ((int)rate > 0) &&
 	    ((int)new_rate != (int)rate))
-	    notice(i18n("The sample rate %1Hz is not supported, "\
-		        "using %2Hz instead.").arg(sr1).arg(sr2));
+	    notice(i18n("%1Hz is not supported, "\
+		        "using %2Hz").arg(sr1).arg(sr2));
     }
     m_dialog->setSupportedSampleRates(supported_rates);
 
@@ -465,8 +500,7 @@ void RecordPlugin::changeSampleRate(double new_rate)
 	const QString sr2(m_dialog->rate2string(rate));
 	if (((int)new_rate > 0) && ((int)rate > 0) &&
 	    ((int)new_rate != (int)rate))
-	    notice(i18n("Setting the sample rate %1Hz failed, "\
-		        "using %2Hz instead.").arg(sr1).arg(sr2));
+	    notice(i18n("%1Hz failed, using %2Hz").arg(sr1).arg(sr2));
     }
     m_dialog->setSampleRate(rate);
 
@@ -481,8 +515,9 @@ void RecordPlugin::changeCompression(int new_compression)
     if (!m_dialog) return;
 
     InhibitRecordGuard _lock(*this); // don't record while settings change
+//     qDebug("RecordPlugin::changeCompression(%d)", (int)new_compression);
 
-    if (!m_device || (new_compression < 0)) {
+    if (!m_device || m_device_name.isNull()) {
 	// no device -> dummy/shortcut
 	m_dialog->setCompression(-1);
 	changeBitsPerSample(0);
@@ -507,8 +542,8 @@ void RecordPlugin::changeCompression(int new_compression)
 	if (compression != new_compression) {
 	    const QString c1(types.name(types.findFromData(new_compression)));
 	    const QString c2(types.name(types.findFromData(compression)));
-	    notice(i18n("The compression '%1' is not supported, "\
-		        "using '%2' instead.").arg(c1).arg(c2));
+	    notice(i18n("compression '%1' not supported, using '%2'").arg(
+                        c1).arg(c2));
 	}
     }
     m_dialog->setSupportedCompressions(supported_comps);
@@ -524,8 +559,7 @@ void RecordPlugin::changeCompression(int new_compression)
 	    const QString c1(types.name(types.findFromData(compression)));
 	    const QString c2(types.name(types.findFromData(
 	                 m_device->compression())));
-	    notice(i18n("Setting the compression type %1 failed, "\
-		        "using %2 instead.").arg(c1).arg(c2));
+	    notice(i18n("compression %1 failed, using %2.").arg(c1).arg(c2));
 	}
     }
     m_dialog->setCompression(compression);
@@ -541,9 +575,9 @@ void RecordPlugin::changeBitsPerSample(unsigned int new_bits)
     if (!m_dialog) return;
 
     InhibitRecordGuard _lock(*this); // don't record while settings change
-//     qDebug("RecordPlugin::changeBitsPerSample(%u)", new_bits);
+//     qDebug("RecordPlugin::changeBitsPerSample(%d)", (int)new_bits);
 
-    if (!m_device || !new_bits) {
+    if (!m_device || m_device_name.isNull()) {
 	// no device -> dummy/shortcut
 	m_dialog->setBitsPerSample(0);
 	changeSampleFormat(SampleFormat::Unknown);
@@ -564,8 +598,8 @@ void RecordPlugin::changeBitsPerSample(unsigned int new_bits)
 	bits = nearest;
 
 	if (((int)new_bits > 0) && (bits > 0)) notice(
-	    i18n("The resolution %1 bits per sample is not supported, "\
-	         "using %2 bits per sample instead.").arg(
+	    i18n("%1 bits per sample is not supported, "\
+	         "using %2 bits per sample").arg(
 		 (int)new_bits).arg(bits));
     }
     m_dialog->setSupportedBits(supported_bits);
@@ -576,8 +610,8 @@ void RecordPlugin::changeBitsPerSample(unsigned int new_bits)
 	// revert to the current device setting if failed
 	bits = m_device->bitsPerSample();
 	if ((new_bits> 0) && (bits > 0)) notice(
-	    i18n("Setting the resolution %1 bits per sample failed, "\
-		 "using %2 bits per sample instead.").arg(
+	    i18n("%1 bits per sample failed, "\
+		 "using %2 bits per sample").arg(
 		 (int)new_bits).arg(bits));
     }
     m_dialog->setBitsPerSample(bits);
@@ -595,7 +629,7 @@ void RecordPlugin::changeSampleFormat(SampleFormat new_format)
     InhibitRecordGuard _lock(*this); // don't record while settings change
 //     qDebug("RecordPlugin::changeSampleFormat(%d)", (int)new_format);
 
-    if (!m_device || (new_format < 0)) {
+    if (!m_device || m_device_name.isNull()) {
 	// no device -> dummy/shortcut
 	m_dialog->setSampleFormat(SampleFormat::Unknown);
 	return;
@@ -618,8 +652,8 @@ void RecordPlugin::changeSampleFormat(SampleFormat new_format)
 	const QString s1 = sf.name(sf.findFromData(new_format));
 	const QString s2 = sf.name(sf.findFromData(format));
 	if (!(new_format == -1) && !(new_format == format)) {
-	    notice(i18n("The sample format '%1' is not supported, "\
-		        "using '%2' instead.").arg(s1).arg(s2));
+	    notice(i18n("sample format '%1' is not supported, "\
+		        "using '%2'").arg(s1).arg(s2));
 	}
     }
     m_dialog->setSupportedSampleFormats(supported_formats);
@@ -634,8 +668,7 @@ void RecordPlugin::changeSampleFormat(SampleFormat new_format)
 	const QString s1 = sf.name(sf.findFromData(new_format));
 	const QString s2 = sf.name(sf.findFromData(format));
 	if (format > 0) notice(
-	    i18n("Setting the sample format '%1' failed, "\
-	         "using '%2' instead.").arg(s1).arg(s2));
+	    i18n("sample format '%1' failed, using '%2'").arg(s1).arg(s2));
     }
     m_dialog->setSampleFormat(format);
 }
@@ -655,7 +688,7 @@ void RecordPlugin::enterInhibit()
 	// set hourglass cursor
 	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
 
-	qDebug("RecordPlugin::enterInhibit() - STOPPING");
+// 	qDebug("RecordPlugin::enterInhibit() - STOPPING");
 	m_thread->stop();
 	Q_ASSERT(!m_thread->running());
     }
@@ -668,37 +701,49 @@ void RecordPlugin::leaveInhibit()
     Q_ASSERT(m_dialog);
 
     if (m_inhibit_count) m_inhibit_count--;
-    if (!m_inhibit_count && m_thread && m_device && m_dialog) {
+
+    while (!m_inhibit_count && paramsValid()) {
 // 	qDebug("RecordPlugin::leaveInhibit() - STARTING ("+
 // 	       "%d channels, %d bits)",
-//                m_dialog->params().tracks,
+// 	       m_dialog->params().tracks,
 // 	       m_dialog->params().bits_per_sample);
 
 	Q_ASSERT(!m_thread->running());
-	if (m_thread->running()) return;
+	if (m_thread->running()) break;
 
 	// set new parameters for the recorder
 	setupRecordThread();
 
 	// and let the thread run (again)
 	m_thread->start();
-
-	// take back the hourglass cursor
-	QApplication::restoreOverrideCursor();
+	break;
     }
+
+    // take back the hourglass cursor
+    if (!m_inhibit_count) QApplication::restoreOverrideCursor();
+}
+
+//***************************************************************************
+bool RecordPlugin::paramsValid()
+{
+    if (!m_thread || !m_device || !m_dialog) return false;
+    if (m_device_name.isNull()) return false;
+
+    const RecordParams &params = m_dialog->params();
+    if (params.tracks < 1) return false;
+
+    return true;
 }
 
 //***************************************************************************
 void RecordPlugin::resetRecording(bool &accepted)
 {
     InhibitRecordGuard _lock(*this);
-    qDebug("RecordPlugin::resetRecording()");
 
     TopWidget &topwidget = this->manager().topWidget();
     accepted = topwidget.closeFile();
     if (!accepted) return;
 
-    m_writers.flush();
     m_writers.resize(0);
     m_buffers_recorded = 0;
 
@@ -712,7 +757,7 @@ void RecordPlugin::setupRecordThread()
     Q_ASSERT(m_thread);
     Q_ASSERT(m_dialog);
     Q_ASSERT(m_device);
-    if (!m_thread || !m_dialog || !m_device) return;
+    if (!paramsValid()) return;
 
     // stop the thread if necessary (should never happen)
     Q_ASSERT(!m_thread->running());
@@ -725,6 +770,7 @@ void RecordPlugin::setupRecordThread()
 
     // our own reference to the record parameters
     const RecordParams &params = m_dialog->params();
+    if (!paramsValid()) return;
 
     // create a decoder for the current sample format
     switch (params.compression) {
@@ -803,7 +849,7 @@ void RecordPlugin::startRecording()
     Q_ASSERT(m_device);
     if (!m_dialog || !m_thread || !m_device) return;
 
-//     InhibitRecordGuard _lock(*this); // don't record while settings change
+    InhibitRecordGuard _lock(*this); // don't record while settings change
     qDebug("RecordPlugin::startRecording()");
 
     if ((m_state != REC_PAUSED) || !m_decoder) {
@@ -904,10 +950,11 @@ void RecordPlugin::recordStopped(int reason)
 //***************************************************************************
 void RecordPlugin::stateChanged(RecordState state)
 {
-    qDebug("RecordPlugin::stateChanged(%d)", (int)state);
+    qDebug("RecordPlugin::stateChanged(%s)", m_controller.stateName(state));
 
     m_state = state;
     switch (m_state) {
+	case REC_UNINITIALIZED:
 	case REC_EMPTY:
 	case REC_PAUSED:
 	case REC_DONE:
@@ -1216,6 +1263,7 @@ void RecordPlugin::processBuffer(QByteArray buffer)
     RecordState state = m_state;
     for (unsigned int track=0; track < tracks; ++track) {
 	switch (state) {
+	    case REC_UNINITIALIZED:
 	    case REC_EMPTY:
 		break;
 	    case REC_BUFFERING:
@@ -1256,5 +1304,12 @@ void RecordPlugin::processBuffer(QByteArray buffer)
 
 }
 
+//***************************************************************************
+void RecordPlugin::prerecordingChanged(bool enable)
+{
+    (void)enable;
+    InhibitRecordGuard _lock(*this); // activate the change
+}
+
 //***************************************************************************
 //***************************************************************************
